@@ -22,7 +22,6 @@ import (
 	"github.com/prometheus/common/model"
 
 	"github.com/prometheus/prometheus/storage/metric"
-	"github.com/prometheus/prometheus/util/strutil"
 )
 
 // Tree returns a string of the tree structure of the given node.
@@ -104,13 +103,14 @@ func (node *AlertStmt) String() string {
 	s := fmt.Sprintf("ALERT %s", node.Name)
 	s += fmt.Sprintf("\n\tIF %s", node.Expr)
 	if node.Duration > 0 {
-		s += fmt.Sprintf("\n\tFOR %s", strutil.DurationToString(node.Duration))
+		s += fmt.Sprintf("\n\tFOR %s", model.Duration(node.Duration))
 	}
 	if len(node.Labels) > 0 {
-		s += fmt.Sprintf("\n\tWITH %s", node.Labels)
+		s += fmt.Sprintf("\n\tLABELS %s", node.Labels)
+	}
+	if len(node.Annotations) > 0 {
+		s += fmt.Sprintf("\n\tANNOTATIONS %s", node.Annotations)
 	}
-	s += fmt.Sprintf("\n\tSUMMARY %q", node.Summary)
-	s += fmt.Sprintf("\n\tDESCRIPTION %q", node.Description)
 	return s
 }
 
@@ -135,13 +135,22 @@ func (es Expressions) String() (s string) {
 }
 
 func (node *AggregateExpr) String() string {
-	aggrString := fmt.Sprintf("%s(%s)", node.Op, node.Expr)
+	aggrString := fmt.Sprintf("%s(", node.Op)
+	if node.Op.isAggregatorWithParam() {
+		aggrString += fmt.Sprintf("%s, ", node.Param)
+	}
+	aggrString += fmt.Sprintf("%s)", node.Expr)
 	if len(node.Grouping) > 0 {
-		format := "%s BY (%s)"
-		if node.KeepExtraLabels {
-			format += " KEEP_COMMON"
+		var format string
+		if node.Without {
+			format = "%s WITHOUT (%s)"
+		} else {
+			format = "%s BY (%s)"
 		}
-		return fmt.Sprintf(format, aggrString, node.Grouping)
+		aggrString = fmt.Sprintf(format, aggrString, node.Grouping)
+	}
+	if node.KeepCommonLabels {
+		aggrString += " KEEP_COMMON"
 	}
 	return aggrString
 }
@@ -154,13 +163,22 @@ func (node *BinaryExpr) String() string {
 
 	matching := ""
 	vm := node.VectorMatching
-	if vm != nil && len(vm.On) > 0 {
-		matching = fmt.Sprintf(" ON(%s)", vm.On)
-		if vm.Card == CardManyToOne {
-			matching += fmt.Sprintf(" GROUP_LEFT(%s)", vm.Include)
+	if vm != nil && len(vm.MatchingLabels) > 0 {
+		if vm.On {
+			matching = fmt.Sprintf(" ON(%s)", vm.MatchingLabels)
+		} else {
+			matching = fmt.Sprintf(" IGNORING(%s)", vm.MatchingLabels)
 		}
-		if vm.Card == CardOneToMany {
-			matching += fmt.Sprintf(" GROUP_RIGHT(%s)", vm.Include)
+		if vm.Card == CardManyToOne || vm.Card == CardOneToMany {
+			matching += " GROUP_"
+			if vm.Card == CardManyToOne {
+				matching += "LEFT"
+			} else {
+				matching += "RIGHT"
+			}
+			if len(vm.Include) > 0 {
+				matching += fmt.Sprintf("(%s)", vm.Include)
+			}
 		}
 	}
 	return fmt.Sprintf("%s %s%s%s %s", node.LHS, node.Op, returnBool, matching, node.RHS)
@@ -177,9 +195,9 @@ func (node *MatrixSelector) String() string {
 	}
 	offset := ""
 	if node.Offset != time.Duration(0) {
-		offset = fmt.Sprintf(" OFFSET %s", strutil.DurationToString(node.Offset))
+		offset = fmt.Sprintf(" OFFSET %s", model.Duration(node.Offset))
 	}
-	return fmt.Sprintf("%s[%s]%s", vecSelector.String(), strutil.DurationToString(node.Range), offset)
+	return fmt.Sprintf("%s[%s]%s", vecSelector.String(), model.Duration(node.Range), offset)
 }
 
 func (node *NumberLiteral) String() string {
@@ -209,7 +227,7 @@ func (node *VectorSelector) String() string {
 	}
 	offset := ""
 	if node.Offset != time.Duration(0) {
-		offset = fmt.Sprintf(" OFFSET %s", strutil.DurationToString(node.Offset))
+		offset = fmt.Sprintf(" OFFSET %s", model.Duration(node.Offset))
 	}
 
 	if len(labelStrings) == 0 {
