@@ -60,6 +60,9 @@ const (
 	requestedPurge     = "purge_on_request"
 	memoryMaintenance  = "maintenance_in_memory"
 	archiveMaintenance = "maintenance_in_archive"
+	completedQurantine = "quarantine_completed"
+	droppedQuarantine  = "quarantine_dropped"
+	failedQuarantine   = "quarantine_failed"
 
 	// Op-types for chunkOps.
 	createAndPin    = "create" // A chunkDesc creation with refCount=1.
@@ -79,6 +82,12 @@ const (
 	// Maintenance types for maintainSeriesDuration.
 	maintainInMemory = "memory"
 	maintainArchived = "archived"
+
+	discardReasonLabel = "reason"
+
+	// Reasons to discard samples.
+	outOfOrderTimestamp = "timestamp_out_of_order"
+	duplicateSample     = "multiple_values_for_timestamp"
 )
 
 func init() {
@@ -89,7 +98,7 @@ func init() {
 
 var (
 	// Global counter, also used internally, so not implemented as
-	// metrics. Collected in memorySeriesStorage.Collect.
+	// metrics. Collected in MemorySeriesStorage.Collect.
 	// TODO(beorn7): As it is used internally, it is actually very bad style
 	// to have it as a global variable.
 	numMemChunks int64
