@@ -162,9 +162,15 @@ type memorySeries struct {
 	// first chunk before its chunk desc is evicted. In doubt, this field is
 	// just set to the oldest possible timestamp.
 	savedFirstTime model.Time
-	// The timestamp of the last sample in this series. Needed for fast access to
-	// ensure timestamp monotonicity during ingestion.
+	// The timestamp of the last sample in this series. Needed for fast
+	// access for federation and to ensure timestamp monotonicity during
+	// ingestion.
 	lastTime model.Time
+	// The last ingested sample value. Needed for fast access for
+	// federation.
+	lastSampleValue model.SampleValue
+	// Whether lastSampleValue has been set already.
+	lastSampleValueSet bool
 	// Whether the current head chunk has already been finished.  If true,
 	// the current head chunk must not be modified anymore.
 	headChunkClosed bool
@@ -185,12 +191,15 @@ type memorySeries struct {
 // set to model.Earliest. The zero value for modTime can be used if the
 // modification time of the series file is unknown (e.g. if this is a genuinely
 // new series).
-func newMemorySeries(m model.Metric, chunkDescs []*chunkDesc, modTime time.Time) *memorySeries {
+func newMemorySeries(m model.Metric, chunkDescs []*chunkDesc, modTime time.Time) (*memorySeries, error) {
+	var err error
 	firstTime := model.Earliest
 	lastTime := model.Earliest
 	if len(chunkDescs) > 0 {
 		firstTime = chunkDescs[0].firstTime()
-		lastTime = chunkDescs[len(chunkDescs)-1].lastTime()
+		if lastTime, err = chunkDescs[len(chunkDescs)-1].lastTime(); err != nil {
+			return nil, err
+		}
 	}
 	return &memorySeries{
 		metric:           m,
@@ -200,16 +209,16 @@ func newMemorySeries(m model.Metric, chunkDescs []*chunkDesc, modTime time.Time)
 		lastTime:         lastTime,
 		persistWatermark: len(chunkDescs),
 		modTime:          modTime,
-	}
+	}, nil
 }
 
 // add adds a sample pair to the series. It returns the number of newly
 // completed chunks (which are now eligible for persistence).
 //
 // The caller must have locked the fingerprint of the series.
-func (s *memorySeries) add(v *model.SamplePair) int {
+func (s *memorySeries) add(v model.SamplePair) (int, error) {
 	if len(s.chunkDescs) == 0 || s.headChunkClosed {
-		newHead := newChunkDesc(newChunk())
+		newHead := newChunkDesc(newChunk(), v.Timestamp)
 		s.chunkDescs = append(s.chunkDescs, newHead)
 		s.headChunkClosed = false
 	} else if s.headChunkUsedByIterator && s.head().refCount() > 1 {
@@ -229,15 +238,25 @@ func (s *memorySeries) add(v *model.SamplePair) int {
 		s.headChunkUsedByIterator = false
 	}
 
-	chunks := s.head().add(v)
+	chunks, err := s.head().add(v)
+	if err != nil {
+		return 0, err
+	}
 	s.head().c = chunks[0]
 
 	for _, c := range chunks[1:] {
-		s.chunkDescs = append(s.chunkDescs, newChunkDesc(c))
+		s.chunkDescs = append(s.chunkDescs, newChunkDesc(c, c.firstTime()))
+	}
+
+	// Populate lastTime of now-closed chunks.
+	for _, cd := range s.chunkDescs[len(s.chunkDescs)-len(chunks) : len(s.chunkDescs)-1] {
+		cd.maybePopulateLastTime()
 	}
 
 	s.lastTime = v.Timestamp
-	return len(chunks) - 1
+	s.lastSampleValue = v.Value
+	s.lastSampleValueSet = true
+	return len(chunks) - 1, nil
 }
 
 // maybeCloseHeadChunk closes the head chunk if it has not been touched for the
@@ -254,6 +273,7 @@ func (s *memorySeries) maybeCloseHeadChunk() bool {
 		// Since we cannot modify the head chunk from now on, we
 		// don't need to bother with cloning anymore.
 		s.headChunkUsedByIterator = false
+		s.head().maybePopulateLastTime()
 		return true
 	}
 	return false
@@ -281,35 +301,46 @@ func (s *memorySeries) evictChunkDescs(iOldestNotEvicted int) {
 
 // dropChunks removes chunkDescs older than t. The caller must have locked the
 // fingerprint of the series.
-func (s *memorySeries) dropChunks(t model.Time) {
+func (s *memorySeries) dropChunks(t model.Time) error {
 	keepIdx := len(s.chunkDescs)
 	for i, cd := range s.chunkDescs {
-		if !cd.lastTime().Before(t) {
+		lt, err := cd.lastTime()
+		if err != nil {
+			return err
+		}
+		if !lt.Before(t) {
 			keepIdx = i
 			break
 		}
 	}
-	if keepIdx > 0 {
-		s.chunkDescs = append(
-			make([]*chunkDesc, 0, len(s.chunkDescs)-keepIdx),
-			s.chunkDescs[keepIdx:]...,
-		)
-		s.persistWatermark -= keepIdx
-		if s.persistWatermark < 0 {
-			panic("dropped unpersisted chunks from memory")
-		}
-		if s.chunkDescsOffset != -1 {
-			s.chunkDescsOffset += keepIdx
-		}
-		numMemChunkDescs.Sub(float64(keepIdx))
-		s.dirty = true
+	if keepIdx == len(s.chunkDescs) && !s.headChunkClosed {
+		// Never drop an open head chunk.
+		keepIdx--
 	}
+	if keepIdx <= 0 {
+		// Nothing to drop.
+		return nil
+	}
+	s.chunkDescs = append(
+		make([]*chunkDesc, 0, len(s.chunkDescs)-keepIdx),
+		s.chunkDescs[keepIdx:]...,
+	)
+	s.persistWatermark -= keepIdx
+	if s.persistWatermark < 0 {
+		panic("dropped unpersisted chunks from memory")
+	}
+	if s.chunkDescsOffset != -1 {
+		s.chunkDescsOffset += keepIdx
+	}
+	numMemChunkDescs.Sub(float64(keepIdx))
+	s.dirty = true
+	return nil
 }
 
 // preloadChunks is an internal helper method.
 func (s *memorySeries) preloadChunks(
-	indexes []int, fp model.Fingerprint, mss *memorySeriesStorage,
-) ([]*chunkDesc, error) {
+	indexes []int, fp model.Fingerprint, mss *MemorySeriesStorage,
+) ([]*chunkDesc, SeriesIterator, error) {
 	loadIndexes := []int{}
 	pinnedChunkDescs := make([]*chunkDesc, 0, len(indexes))
 	for _, idx := range indexes {
@@ -333,52 +364,81 @@ func (s *memorySeries) preloadChunks(
 				cd.unpin(mss.evictRequests)
 			}
 			chunkOps.WithLabelValues(unpin).Add(float64(len(pinnedChunkDescs)))
-			return nil, err
+			return nil, nopIter, err
 		}
 		for i, c := range chunks {
 			s.chunkDescs[loadIndexes[i]].setChunk(c)
 		}
 	}
-	return pinnedChunkDescs, nil
-}
 
-/*
-func (s *memorySeries) preloadChunksAtTime(t model.Time, p *persistence) (chunkDescs, error) {
-	s.mtx.Lock()
-	defer s.mtx.Unlock()
-
-	if len(s.chunkDescs) == 0 {
-		return nil, nil
+	if !s.headChunkClosed && indexes[len(indexes)-1] == len(s.chunkDescs)-1 {
+		s.headChunkUsedByIterator = true
 	}
 
-	var pinIndexes []int
-	// Find first chunk where lastTime() is after or equal to t.
-	i := sort.Search(len(s.chunkDescs), func(i int) bool {
-		return !s.chunkDescs[i].lastTime().Before(t)
-	})
-	switch i {
-	case 0:
-		pinIndexes = []int{0}
-	case len(s.chunkDescs):
-		pinIndexes = []int{i - 1}
-	default:
-		if s.chunkDescs[i].contains(t) {
-			pinIndexes = []int{i}
-		} else {
-			pinIndexes = []int{i - 1, i}
+	curriedQuarantineSeries := func(err error) {
+		mss.quarantineSeries(fp, s.metric, err)
+	}
+
+	iter := &boundedIterator{
+		it:    s.newIterator(pinnedChunkDescs, curriedQuarantineSeries),
+		start: model.Now().Add(-mss.dropAfter),
+	}
+
+	return pinnedChunkDescs, iter, nil
+}
+
+// newIterator returns a new SeriesIterator for the provided chunkDescs (which
+// must be pinned).
+//
+// The caller must have locked the fingerprint of the memorySeries.
+func (s *memorySeries) newIterator(pinnedChunkDescs []*chunkDesc, quarantine func(error)) SeriesIterator {
+	chunks := make([]chunk, 0, len(pinnedChunkDescs))
+	for _, cd := range pinnedChunkDescs {
+		// It's OK to directly access cd.c here (without locking) as the
+		// series FP is locked and the chunk is pinned.
+		chunks = append(chunks, cd.c)
+	}
+	return &memorySeriesIterator{
+		chunks:     chunks,
+		chunkIts:   make([]chunkIterator, len(chunks)),
+		quarantine: quarantine,
+	}
+}
+
+// preloadChunksForInstant preloads chunks for the latest value in the given
+// range. If the last sample saved in the memorySeries itself is the latest
+// value in the given range, it will in fact preload zero chunks and just take
+// that value.
+func (s *memorySeries) preloadChunksForInstant(
+	fp model.Fingerprint,
+	from model.Time, through model.Time,
+	mss *MemorySeriesStorage,
+) ([]*chunkDesc, SeriesIterator, error) {
+	// If we have a lastSamplePair in the series, and thas last samplePair
+	// is in the interval, just take it in a singleSampleSeriesIterator. No
+	// need to pin or load anything.
+	lastSample := s.lastSamplePair()
+	if !through.Before(lastSample.Timestamp) &&
+		!from.After(lastSample.Timestamp) &&
+		lastSample != ZeroSamplePair {
+		iter := &boundedIterator{
+			it:    &singleSampleSeriesIterator{samplePair: lastSample},
+			start: model.Now().Add(-mss.dropAfter),
 		}
+		return nil, iter, nil
 	}
-
-	return s.preloadChunks(pinIndexes, p)
+	// If we are here, we are out of luck and have to delegate to the more
+	// expensive method.
+	return s.preloadChunksForRange(fp, from, through, mss)
 }
-*/
 
 // preloadChunksForRange loads chunks for the given range from the persistence.
 // The caller must have locked the fingerprint of the series.
 func (s *memorySeries) preloadChunksForRange(
+	fp model.Fingerprint,
 	from model.Time, through model.Time,
-	fp model.Fingerprint, mss *memorySeriesStorage,
-) ([]*chunkDesc, error) {
+	mss *MemorySeriesStorage,
+) ([]*chunkDesc, SeriesIterator, error) {
 	firstChunkDescTime := model.Latest
 	if len(s.chunkDescs) > 0 {
 		firstChunkDescTime = s.chunkDescs[0].firstTime()
@@ -386,15 +446,16 @@ func (s *memorySeries) preloadChunksForRange(
 	if s.chunkDescsOffset != 0 && from.Before(firstChunkDescTime) {
 		cds, err := mss.loadChunkDescs(fp, s.persistWatermark)
 		if err != nil {
-			return nil, err
+			return nil, nopIter, err
 		}
 		s.chunkDescs = append(cds, s.chunkDescs...)
 		s.chunkDescsOffset = 0
 		s.persistWatermark += len(cds)
+		firstChunkDescTime = s.chunkDescs[0].firstTime()
 	}
 
-	if len(s.chunkDescs) == 0 {
-		return nil, nil
+	if len(s.chunkDescs) == 0 || through.Before(firstChunkDescTime) {
+		return nil, nopIter, nil
 	}
 
 	// Find first chunk with start time after "from".
@@ -405,6 +466,17 @@ func (s *memorySeries) preloadChunksForRange(
 	throughIdx := sort.Search(len(s.chunkDescs), func(i int) bool {
 		return s.chunkDescs[i].firstTime().After(through)
 	})
+	if fromIdx == len(s.chunkDescs) {
+		// Even the last chunk starts before "from". Find out if the
+		// series ends before "from" and we don't need to do anything.
+		lt, err := s.chunkDescs[len(s.chunkDescs)-1].lastTime()
+		if err != nil {
+			return nil, nopIter, err
+		}
+		if lt.Before(from) {
+			return nil, nopIter, nil
+		}
+	}
 	if fromIdx > 0 {
 		fromIdx--
 	}
@@ -419,25 +491,6 @@ func (s *memorySeries) preloadChunksForRange(
 	return s.preloadChunks(pinIndexes, fp, mss)
 }
 
-// newIterator returns a new SeriesIterator. The caller must have locked the
-// fingerprint of the memorySeries.
-func (s *memorySeries) newIterator() SeriesIterator {
-	chunks := make([]chunk, 0, len(s.chunkDescs))
-	for i, cd := range s.chunkDescs {
-		if chunk := cd.chunk(); chunk != nil {
-			if i == len(s.chunkDescs)-1 && !s.headChunkClosed {
-				s.headChunkUsedByIterator = true
-			}
-			chunks = append(chunks, chunk)
-		}
-	}
-
-	return &memorySeriesIterator{
-		chunks:   chunks,
-		chunkIts: make([]chunkIterator, len(chunks)),
-	}
-}
-
 // head returns a pointer to the head chunk descriptor. The caller must have
 // locked the fingerprint of the memorySeries. This method will panic if this
 // series has no chunk descriptors.
@@ -445,8 +498,9 @@ func (s *memorySeries) head() *chunkDesc {
 	return s.chunkDescs[len(s.chunkDescs)-1]
 }
 
-// firstTime returns the timestamp of the first sample in the series. The caller
-// must have locked the fingerprint of the memorySeries.
+// firstTime returns the timestamp of the first sample in the series.
+//
+// The caller must have locked the fingerprint of the memorySeries.
 func (s *memorySeries) firstTime() model.Time {
 	if s.chunkDescsOffset == 0 && len(s.chunkDescs) > 0 {
 		return s.chunkDescs[0].firstTime()
@@ -454,6 +508,23 @@ func (s *memorySeries) firstTime() model.Time {
 	return s.savedFirstTime
 }
 
+// lastSamplePair returns the last ingested SamplePair. It returns
+// ZeroSamplePair if this memorySeries has never received a sample (via the add
+// method), which is the case for freshly unarchived series or newly created
+// ones and also for all series after a server restart. However, in that case,
+// series will most likely be considered stale anyway.
+//
+// The caller must have locked the fingerprint of the memorySeries.
+func (s *memorySeries) lastSamplePair() model.SamplePair {
+	if !s.lastSampleValueSet {
+		return ZeroSamplePair
+	}
+	return model.SamplePair{
+		Timestamp: s.lastTime,
+		Value:     s.lastSampleValue,
+	}
+}
+
 // chunksToPersist returns a slice of chunkDescs eligible for persistence. It's
 // the caller's responsibility to actually persist the returned chunks
 // afterwards. The method sets the persistWatermark and the dirty flag
@@ -476,129 +547,53 @@ func (s *memorySeries) chunksToPersist() []*chunkDesc {
 
 // memorySeriesIterator implements SeriesIterator.
 type memorySeriesIterator struct {
-	chunkIt  chunkIterator   // Last chunkIterator used by ValueAtTime.
-	chunkIts []chunkIterator // Caches chunkIterators.
-	chunks   []chunk
+	chunkIt    chunkIterator   // Last chunkIterator used by ValueAtOrBeforeTime.
+	chunkIts   []chunkIterator // Caches chunkIterators.
+	chunks     []chunk
+	quarantine func(error) // Call to quarantine the series this iterator belongs to.
 }
 
-// ValueAtTime implements SeriesIterator.
-func (it *memorySeriesIterator) ValueAtTime(t model.Time) []model.SamplePair {
+// ValueAtOrBeforeTime implements SeriesIterator.
+func (it *memorySeriesIterator) ValueAtOrBeforeTime(t model.Time) model.SamplePair {
 	// The most common case. We are iterating through a chunk.
-	if it.chunkIt != nil && it.chunkIt.contains(t) {
-		return it.chunkIt.valueAtTime(t)
+	if it.chunkIt != nil {
+		containsT, err := it.chunkIt.contains(t)
+		if err != nil {
+			it.quarantine(err)
+			return ZeroSamplePair
+		}
+		if containsT {
+			if it.chunkIt.findAtOrBefore(t) {
+				return it.chunkIt.value()
+			}
+			if it.chunkIt.err() != nil {
+				it.quarantine(it.chunkIt.err())
+			}
+			return ZeroSamplePair
+		}
 	}
 
 	if len(it.chunks) == 0 {
-		return nil
+		return ZeroSamplePair
 	}
 
-	// Before or exactly on the first sample of the series.
-	it.chunkIt = it.chunkIterator(0)
-	ts := it.chunkIt.timestampAtIndex(0)
-	if !t.After(ts) {
-		// return first value of first chunk
-		return []model.SamplePair{{
-			Timestamp: ts,
-			Value:     it.chunkIt.sampleValueAtIndex(0),
-		}}
-	}
-
-	// After or exactly on the last sample of the series.
-	it.chunkIt = it.chunkIterator(len(it.chunks) - 1)
-	ts = it.chunkIt.lastTimestamp()
-	if !t.Before(ts) {
-		// return last value of last chunk
-		return []model.SamplePair{{
-			Timestamp: ts,
-			Value:     it.chunkIt.sampleValueAtIndex(it.chunkIt.length() - 1),
-		}}
-	}
-
-	// Find last chunk where firstTime() is before or equal to t.
+	// Find the last chunk where firstTime() is before or equal to t.
 	l := len(it.chunks) - 1
 	i := sort.Search(len(it.chunks), func(i int) bool {
 		return !it.chunks[l-i].firstTime().After(t)
 	})
 	if i == len(it.chunks) {
-		panic("out of bounds")
+		// Even the first chunk starts after t.
+		return ZeroSamplePair
 	}
 	it.chunkIt = it.chunkIterator(l - i)
-	ts = it.chunkIt.lastTimestamp()
-	if t.After(ts) {
-		// We ended up between two chunks.
-		sp1 := model.SamplePair{
-			Timestamp: ts,
-			Value:     it.chunkIt.sampleValueAtIndex(it.chunkIt.length() - 1),
-		}
-		it.chunkIt = it.chunkIterator(l - i + 1)
-		return []model.SamplePair{
-			sp1,
-			{
-				Timestamp: it.chunkIt.timestampAtIndex(0),
-				Value:     it.chunkIt.sampleValueAtIndex(0),
-			},
-		}
+	if it.chunkIt.findAtOrBefore(t) {
+		return it.chunkIt.value()
 	}
-	return it.chunkIt.valueAtTime(t)
-}
-
-// BoundaryValues implements SeriesIterator.
-func (it *memorySeriesIterator) BoundaryValues(in metric.Interval) []model.SamplePair {
-	// Find the first chunk for which the first sample is within the interval.
-	i := sort.Search(len(it.chunks), func(i int) bool {
-		return !it.chunks[i].firstTime().Before(in.OldestInclusive)
-	})
-	// Only now check the last timestamp of the previous chunk (which is
-	// fairly expensive).
-	if i > 0 && !it.chunkIterator(i-1).lastTimestamp().Before(in.OldestInclusive) {
-		i--
+	if it.chunkIt.err() != nil {
+		it.quarantine(it.chunkIt.err())
 	}
-
-	values := make([]model.SamplePair, 0, 2)
-	for j, c := range it.chunks[i:] {
-		if c.firstTime().After(in.NewestInclusive) {
-			if len(values) == 1 {
-				// We found the first value before but are now
-				// already past the last value. The value we
-				// want must be the last value of the previous
-				// chunk. So backtrack...
-				chunkIt := it.chunkIterator(i + j - 1)
-				values = append(values, model.SamplePair{
-					Timestamp: chunkIt.lastTimestamp(),
-					Value:     chunkIt.lastSampleValue(),
-				})
-			}
-			break
-		}
-		chunkIt := it.chunkIterator(i + j)
-		if len(values) == 0 {
-			firstValues := chunkIt.valueAtTime(in.OldestInclusive)
-			switch len(firstValues) {
-			case 2:
-				values = append(values, firstValues[1])
-			case 1:
-				values = firstValues
-			default:
-				panic("unexpected return from valueAtTime")
-			}
-		}
-		if chunkIt.lastTimestamp().After(in.NewestInclusive) {
-			values = append(values, chunkIt.valueAtTime(in.NewestInclusive)[0])
-			break
-		}
-	}
-	if len(values) == 1 {
-		// We found exactly one value. In that case, add the most recent we know.
-		chunkIt := it.chunkIterator(len(it.chunks) - 1)
-		values = append(values, model.SamplePair{
-			Timestamp: chunkIt.lastTimestamp(),
-			Value:     chunkIt.lastSampleValue(),
-		})
-	}
-	if len(values) == 2 && values[0].Equal(&values[1]) {
-		return values[:1]
-	}
-	return values
+	return ZeroSamplePair
 }
 
 // RangeValues implements SeriesIterator.
@@ -609,8 +604,15 @@ func (it *memorySeriesIterator) RangeValues(in metric.Interval) []model.SamplePa
 	})
 	// Only now check the last timestamp of the previous chunk (which is
 	// fairly expensive).
-	if i > 0 && !it.chunkIterator(i-1).lastTimestamp().Before(in.OldestInclusive) {
-		i--
+	if i > 0 {
+		lt, err := it.chunkIterator(i - 1).lastTimestamp()
+		if err != nil {
+			it.quarantine(err)
+			return nil
+		}
+		if !lt.Before(in.OldestInclusive) {
+			i--
+		}
 	}
 
 	values := []model.SamplePair{}
@@ -618,7 +620,12 @@ func (it *memorySeriesIterator) RangeValues(in metric.Interval) []model.SamplePa
 		if c.firstTime().After(in.NewestInclusive) {
 			break
 		}
-		values = append(values, it.chunkIterator(i+j).rangeValues(in)...)
+		chValues, err := rangeValues(it.chunkIterator(i+j), in)
+		if err != nil {
+			it.quarantine(err)
+			return nil
+		}
+		values = append(values, chValues...)
 	}
 	return values
 }
@@ -634,20 +641,41 @@ func (it *memorySeriesIterator) chunkIterator(i int) chunkIterator {
 	return chunkIt
 }
 
+// singleSampleSeriesIterator implements Series Iterator. It is a "shortcut
+// iterator" that returns a single samplee only. The sample is saved in the
+// iterator itself, so no chunks need to be pinned.
+type singleSampleSeriesIterator struct {
+	samplePair model.SamplePair
+}
+
+// ValueAtTime implements SeriesIterator.
+func (it *singleSampleSeriesIterator) ValueAtOrBeforeTime(t model.Time) model.SamplePair {
+	if it.samplePair.Timestamp.After(t) {
+		return ZeroSamplePair
+	}
+	return it.samplePair
+}
+
+// RangeValues implements SeriesIterator.
+func (it *singleSampleSeriesIterator) RangeValues(in metric.Interval) []model.SamplePair {
+	if it.samplePair.Timestamp.After(in.NewestInclusive) ||
+		it.samplePair.Timestamp.Before(in.OldestInclusive) {
+		return []model.SamplePair{}
+	}
+	return []model.SamplePair{it.samplePair}
+}
+
 // nopSeriesIterator implements Series Iterator. It never returns any values.
 type nopSeriesIterator struct{}
 
 // ValueAtTime implements SeriesIterator.
-func (i nopSeriesIterator) ValueAtTime(t model.Time) []model.SamplePair {
-	return []model.SamplePair{}
-}
-
-// BoundaryValues implements SeriesIterator.
-func (i nopSeriesIterator) BoundaryValues(in metric.Interval) []model.SamplePair {
-	return []model.SamplePair{}
+func (i nopSeriesIterator) ValueAtOrBeforeTime(t model.Time) model.SamplePair {
+	return ZeroSamplePair
 }
 
 // RangeValues implements SeriesIterator.
 func (i nopSeriesIterator) RangeValues(in metric.Interval) []model.SamplePair {
 	return []model.SamplePair{}
 }
+
+var nopIter nopSeriesIterator // A nopSeriesIterator for convenience. Can be shared.
